library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_unsigned.ALL;

entity I2S_CONV1_RTL is
   port (
      SCLK    : in std_logic;
      LRCK    : in std_logic;
      SDATA   : in std_logic;
      LEL     : out std_logic;
      LER     : out std_logic;
      SCKL    : out std_logic;
      SCKR    : out std_logic;
      DATAL   : out std_logic;
      DATAR   : out std_logic
      );
end I2S_CONV1_RTL;

architecture RTL of I2S_CONV1_RTL is

   signal sreg_data   : std_logic_vector(31 downto 0);
   signal pdatal      : std_logic_vector(15 downto 0);
   signal pdatar      : std_logic_vector(15 downto 0);
   signal regl        : std_logic_vector(15 downto 0);
   signal regr        : std_logic_vector(15 downto 0);
   signal lrck_d      : std_logic;
   signal lrck_dd     : std_logic;
   signal cnt         : std_logic_vector(3 downto 0);
   signal strobe      : std_logic;
   signal mask        : std_logic;
   signal latch       : std_logic;
   signal scko        : std_logic;

begin

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         lrck_d <= LRCK;
         lrck_dd <= lrck_d;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if (lrck_d = '1' and  lrck_dd = '0') then
            strobe <= '0';
         else
            strobe <= not strobe;
         end if;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         sreg_data <= sreg_data(30 downto 0) & SDATA;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if (lrck_d = '1' and  lrck_dd = '0') then
            pdatal <= sreg_data(31 downto 16);
         end if;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if (lrck_d = '0' and  lrck_dd = '1') then
            pdatar <= sreg_data(31 downto 16);
         end if;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if (strobe = '1') then
            if (lrck_d = '1' and  lrck_dd = '0') then
               regl <= pdatal;
               regr <= pdatar;
            else
               regl <= regl(14 downto 0) & '0';
               regr <= regr(14 downto 0) & '0';
            end if;
         end if;
      end if;
   end process;

   DATAL <= regl(15);
   DATAR <= regr(15);

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if (strobe = '1') then
            latch <= lrck_d  and (not lrck_dd);
         end if;
      end if;
   end process;

   LEL <= not latch;
   LER <= not latch;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if (lrck_d = '0' and lrck_dd = '1') then
            cnt <= (others => '0');
         elsif (cnt = "1111" or strobe = '0') then
            cnt <= cnt;
         else
            cnt <= cnt + '1';
         end if;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if (strobe = '1') then
            if (cnt = "0001") then
               mask <= '1';
            elsif (cnt = "1111") then
               mask <= '0';
            end if;
         end if;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if ((lrck_d = '1' and  lrck_dd = '0') or mask = '1') then
            scko <= '0';
         else
            scko <= not scko;
         end if;
      end if;
   end process;

   SCKL <= scko;
   SCKR <= scko;

end RTL;
