library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_unsigned.ALL;

entity I2S_CONV3_RTL is
   port (
      MCLK    : in std_logic;		-- x512fs
      SCLK    : in std_logic;		-- x64fs
      LRCK    : in std_logic;		-- x1fs
      SDATA   : in std_logic;
      SPDIF   : out std_logic		-- x1fs 
      );
end I2S_CONV3_RTL;

architecture RTL of I2S_CONV3_RTL is

   signal y1          : std_logic;
   signal y2          : std_logic;
   signal sclk_d      : std_logic;
   signal x2sclk      : std_logic;
   signal sreg_data   : std_logic_vector(31 downto 0);
   signal pdata       : std_logic_vector(23 downto 0);
   signal lrck_d      : std_logic;
   signal lrck_dd     : std_logic;
   signal par         : std_logic;
   signal par_mux     : std_logic_vector(7 downto 0);
   signal pdata_sync  : std_logic_vector(31 downto 0);
   signal lrdata      : std_logic;
   signal spdata      : std_logic;
   signal cnt         : std_logic_vector(3 downto 0);

begin

--======================== generate double rate frequency clock of sclk ===============--
   process(MCLK)
   begin
      if (MCLK'event and MCLK='0') then	------ falling edge!!!
         sclk_d <= SCLK;
      end if;
   end process;

   process(MCLK)
   begin
      if (MCLK'event and MCLK='0') then	------ falling edge!!!
         if ((SCLK xnor sclk_d) = '0') then
            y1 <= '1';
         else
            y1 <= not y1;
         end if;
      end if;
   end process;

   y2 <= (not x2sclk) when y1 = '1' else x2sclk;

   process(MCLK)
   begin
      if (MCLK'event and MCLK='0') then	------ falling edge!!!
         if ((SCLK xnor sclk_d) = '0') then
            x2sclk <= '1';
         else
            x2sclk <= y2;
         end if;
      end if;
   end process;
--==================================================================================--

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         sreg_data <= sreg_data(30 downto 0) & SDATA;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if ((lrck_d xor lrck_dd) = '1') then
            pdata <= sreg_data(31 downto 8);
         end if;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         lrck_d <= LRCK;
         lrck_dd <= lrck_d;
      end if;
   end process;

   process(SCLK)
   begin
      if (SCLK'event and SCLK='1') then
         if ((lrck_d xor lrck_dd) = '1') then
            par <= pdata(0) xor pdata(1) xor pdata(2) xor pdata(3) xor pdata(4) xor
                   pdata(5) xor pdata(6) xor pdata(7) xor pdata(8) xor pdata(9) xor
                   pdata(10) xor pdata(11) xor pdata(12) xor pdata(13) xor pdata(14) xor
                   pdata(15) xor pdata(16) xor pdata(17) xor pdata(18) xor pdata(19) xor
                   pdata(20) xor pdata(21) xor pdata(22) xor pdata(23);
         end if;
      end if;
   end process;

   par_mux <= "1000" & par & "000" when LRCK = '0' else		-- for X (Lch)
              "0100" & par & "000";				-- for Y (Rch)

   process(x2sclk)
   begin
      if (x2sclk'event and x2sclk='1') then
         if (((lrck_d xor lrck_dd) and SCLK) = '1') then		-- Load
            pdata_sync(31 downto 8) <= pdata;
            pdata_sync(7 downto 0) <= par_mux;
         elsif (SCLK = '1') then					-- Shift
            pdata_sync <= '0' & pdata_sync(31 downto 1);
         end if;
      end if;
   end process;

   lrdata <= pdata_sync(0);

   process(x2sclk)
   begin
      if (x2sclk'event and x2sclk='1') then
         if ((SCLK = '1') and ((lrck_d xor lrck_dd) = '1')) then
            cnt <= (others => '0');
         elsif (cnt = "1111") then
            cnt <= (others => '1');
         else
            cnt <= cnt + '1';
         end if;
      end if;
   end process;

   process(x2sclk)
   begin
      if (x2sclk'event and x2sclk='1') then
         if ((cnt(3 downto 1) = "101") or (cnt(3 downto 1) = "101")) then
            spdata <= '0';
         elsif ((cnt = "0111") or (cnt = "1000") or (cnt = "1001")) then
            spdata <= '1';
         else
            if (SCLK = '1') then
               spdata <= '1' xor spdata;
            else
               spdata <= lrdata xor spdata;
            end if;
         end if;
      end if;
   end process;

   SPDIF <= spdata;

end RTL;
